//+FHDR--------------------------------------------------------------------------------------------------------
//  Company: 
//-----------------------------------------------------------------------------------------------------------------
//  File Name             :    IIR_Filter.v
//  Department            :    
//  Author                :    thfu
//  Author's Tel          :     
//-----------------------------------------------------------------------------------------------------------------
//  Relese History
//  Version     Date            Author          Description
//  0.4         2024-05-28      thfu
//2024-05-28 10:22:49 
//-----------------------------------------------------------------------------------------------------------------
//  Keywords            :       
//
//-----------------------------------------------------------------------------------------------------------------
//  Parameter
//
//-----------------------------------------------------------------------------------------------------------------
//  Purpose                 :
//                      
//-----------------------------------------------------------------------------------------------------------------
//  Target Device:        
//  Tool versions:        
//-----------------------------------------------------------------------------------------------------------------
//  Reuse Issues
//  Reset Strategy: 
//  Clock Domains: 
//  Critical Timing:
//  Asynchronous I/F:
//  Synthesizable (y/n): 
//  Other:
//-FHDR--------------------------------------------------------------------------------------------------------
module  IIR_Filter_p8 #(
 parameter  data_in_width       = 16 
,parameter  coef_width          = 32 
,parameter  frac_data_out_width = 20//X for in,5
,parameter  frac_coef_width     = 31//division
)
(
 input	 rstn
,input	 clk
,input	 en
,input   signed	[data_in_width-1:0]   dinp0
,input   signed	[data_in_width-1:0]   dinp1
,input   signed	[data_in_width-1:0]   dinp2
,input   signed	[data_in_width-1:0]   dinp3
,input   signed	[data_in_width-1:0]   dinp4
,input   signed	[data_in_width-1:0]   dinp5
,input   signed	[data_in_width-1:0]   dinp6
,input   signed	[data_in_width-1:0]   dinp7

,input   signed	[coef_width-1   :0]   a_re
,input   signed	[coef_width-1   :0]   a_im
,input   signed	[coef_width-1   :0]   ab_re
,input   signed	[coef_width-1   :0]   ab_im
,input   signed	[coef_width-1   :0]   abb_re
,input   signed	[coef_width-1   :0]   abb_im
,input   signed	[coef_width-1   :0]   ab_pow3_re
,input   signed	[coef_width-1   :0]   ab_pow3_im
,input   signed	[coef_width-1   :0]   ab_pow4_re
,input   signed	[coef_width-1   :0]   ab_pow4_im
,input   signed	[coef_width-1   :0]   ab_pow5_re
,input   signed	[coef_width-1   :0]   ab_pow5_im
,input   signed	[coef_width-1   :0]   ab_pow6_re
,input   signed	[coef_width-1   :0]   ab_pow6_im
,input   signed	[coef_width-1   :0]   ab_pow7_re
,input   signed	[coef_width-1   :0]   ab_pow7_im

,input   signed	[coef_width-1   :0]   b_pow8_re
,input   signed	[coef_width-1   :0]   b_pow8_im
,output  signed	[data_in_width-1:0]   dout
);

wire   signed	[data_in_width-1   :0]   dinp  [7:0] = {dinp7,dinp6,dinp5,dinp4,dinp3,dinp2,dinp1,dinp0};
wire   signed	[coef_width-1   :0]   ab_pow_re  [7:0] = {ab_pow7_re, ab_pow6_re,ab_pow5_re,ab_pow4_re,ab_pow3_re,abb_re,ab_re,a_re};
wire   signed	[coef_width-1   :0]   ab_pow_im  [7:0] = {ab_pow7_im, ab_pow6_im,ab_pow5_im,ab_pow4_im,ab_pow3_im,abb_im,ab_im,a_im};

wire signed [data_in_width+frac_data_out_width-1:0] x_re [0:7];
wire signed [data_in_width+frac_data_out_width-1:0] x_im [0:7];

genvar i;
generate
    for (i = 0; i < 8; i = i + 1) begin: mult_c_inst
        mult_C #(
            .A_width(data_in_width),
            .B_width(data_in_width),
            .C_width(coef_width+frac_data_out_width),
            .D_width(coef_width+frac_data_out_width),
            .frac_coef_width(frac_coef_width)
        ) inst_c (
            .clk    (clk),
            .rstn   (rstn),
            .en     (en),
            .a      (dinp[i]),         
            .b      (16'b0),
            .c      ({ab_pow_re[i],{frac_data_out_width{1'b0}}}),
            .d      ({ab_pow_im[i],{frac_data_out_width{1'b0}}}),
            .Re     (x_re[i]),        
            .Im     (x_im[i])
        );
    end
endgenerate

wire	signed	[data_in_width+frac_data_out_width+3:0]  v_re;
wire	signed	[data_in_width+frac_data_out_width+3:0]	 v_im;

assign	v_re       =   x_re[0] + x_re[1] +x_re[2] +x_re[3] +x_re[4] +x_re[5] +x_re[6] +x_re[7];
assign	v_im       =   x_im[0] + x_im[1] +x_im[2] +x_im[3] +x_im[4] +x_im[5] +x_im[6] +x_im[7];

reg	signed	[data_in_width+frac_data_out_width+3:0]  v1_re;
reg	signed	[data_in_width+frac_data_out_width+3:0]	 v1_im;

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
                v1_re    <=    'h0;
                v1_im    <=    'h0;
 		 end 
	 else if(en)
		 begin
                v1_re    <=    v_re;
                v1_im    <=    v_im;
		  end
	 else
		 begin
                v1_re    <=    v1_re;
                v1_im    <=    v1_im;
		  end

wire    signed	[data_in_width+frac_data_out_width+3:0]	 y_re;
wire    signed	[data_in_width+frac_data_out_width+3:0]	 y_im;
wire    signed	[data_in_width+frac_data_out_width+3:0]	 y1_re;
wire    signed	[data_in_width+frac_data_out_width+3:0]	 y1_im;

reg 	signed	[data_in_width-1:0]   dout_re;

mult_C
#(
 .A_width(data_in_width+frac_data_out_width+4) 
,.B_width(data_in_width+frac_data_out_width+4) 
,.C_width(coef_width) 
,.D_width(coef_width) 
,.frac_coef_width(frac_coef_width) 
)
inst_c9         (
                                            .clk	(clk    	),
                                            .rstn	(rstn   	),
                                            .en		(en     	),
                                            .a		(y_re		),
                                            .b		(y_im		),
                                            .c		(b_pow8_re	),
                                            .d		(b_pow8_im	),
                                            .Re		(y1_re     	),//b^8*y(n-1)
                                            .Im		(y1_im     	)
		);

assign	y_re       =   v1_re + y1_re;
assign	y_im       =   v1_im + y1_im;

wire signed	[data_in_width+frac_data_out_width+3:0] dout_round;

FixRound #(data_in_width+frac_data_out_width+4,frac_data_out_width) u_round1 (clk, rstn, en, y_re, dout_round);

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    dout_re  <= 'h0;
		 end 
	 else if(en)
		 begin
		    dout_re  <= dout_round[frac_data_out_width+15:frac_data_out_width];
		  end
	 else
		 begin
		    dout_re  <= dout_re;
		  end

reg 	signed	[data_in_width-1:0]   dout_clip;

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    dout_clip <= 'h0;
		 end 
	 else if(en)
		 begin
	         if(dout_round[frac_data_out_width+16:frac_data_out_width+15]==2'b01)
			dout_clip	<=	16'd32767;
		 else if(dout_round[frac_data_out_width+16:frac_data_out_width+15]==2'b10)
			dout_clip	<=	-16'd32768;
		 else
			dout_clip	<=	dout_re;		 
		 end
	 else
		 begin
		    dout_clip  <= dout_clip;
		  end

assign	dout = dout_clip;

endmodule

